class Mineral
  getter name, hardness

  def initialize(@name : String, @hardness : Float64)
  end

  # Good
  def to_s(io)
    io << name << ", " << hardness
  end

end

min1 = Mineral.new("gold", 42.0)
io = IO::Memory.new
# To see what io contains, use to_s:
min1.to_s(io).to_s # => "gold, 42.0"

# Exceptions:
class CoolException < Exception
end

raise CoolException.new("Somebody pushed the red button")
# => Somebody pushed the red button (CoolException)

ex = begin
  raise CoolException.new
rescue ex1 : IndexError
  ex1.message
rescue ex2 : CoolException | KeyError
  ex2.message
rescue ex3 : Exception
  ex3.message
rescue # catch any kind of exception
  "an unknown exception"
end # => "ex2"

require "json"
path = "path/to/file"

begin
  if File.exists?(path)
    raw_file = File.read(path)
    map = JSON.parse(raw_file)
    File.write(path, "ok")
    :ok
  end
rescue JSON::ParseException # Parsing error
  raise "Could not parse file"
rescue ex
  raise "Other error: #{ex.message}"
end


class MineralC
  def initialize
    @callbacks = [] of ->
  end

  def after_save(&block)
    @callbacks << block
  end

  # save in database, then execute callbacks
  def save
    # save


  rescue ex
    p "Exception occurred: #{ex.message}"
  else
    @callbacks.each &.call
  end
end

min = MineralC.new
min.after_save { puts "Save in DB successful" }
min.after_save { puts "Logging save" }
min.after_save { puts "Replicate save to failover node" }
min.save # =>
# Save in DB successful
# Logging save
# Replicate save to failover node
