require "kemal"
require "sqlite3"

require "./db_json/*"

db = DB.open "sqlite3://../chinook.db"

def table_names(db)
  sql = "SELECT name FROM sqlite_master WHERE type='table';"
  db.query_all(sql, as: String)
end

def write_json(io, col_names, rs)
  JSON.build(io) do |json| # (1)
    json.object do
      col_names.each do |col|
        json_encode_field json, col, rs.read # (2)
      end
    end
  end
  io << "\n"
end


def json_encode_field(json, col, value)
  case value
  when Bytes # (3)
    json.field col do
      json.array do # writes begin and end of an array
        value.each do |e|
          json.scalar e # writes the value
        end
      end
    end
  else
    json.field col do     # write an object's field and value
      value.to_json(json) # (4)
    end
  end
end


# routing handlers:
get "/" do |env|
  env.response.content_type = "application/json"
  tables = table_names(db)
  tables.to_json
end

# show tables in a view:
# get "/" do |env|
#   tables = table_names(db)
#   render "src/views/tables.ecr"
# end

get "/:table_name" do |env|
  env.response.content_type = "application/json"
  table_name = env.params.url["table_name"] # (5)
  # avoid SQL injection by checking table name
  unless table_names(db).includes?(table_name) # (6)
    # ignore if the requested table does not exist.
    env.response.status_code = 404
  else
    db.query "select * from #{table_name}" do |rs| # (7)
      col_names = rs.column_names
      rs.each do                                       # (8)
        write_json(env.response.output, col_names, rs) # (9)
        # force chunked response even on small tables
        env.response.output.flush # (10)
      end
    end
  end
end

Kemal.run

db.close
